==> over_react-3.7.0.tar.gz: tools/analyzer_plugin/lib/src/outline/element_outline.dart <==
import 'package:analyzer_plugin/utilities/analyzer_converter.dart';
import 'package:analyzer_plugin/utilities/outline/outline.dart';
import 'package:analyzer_plugin/protocol/protocol_common.dart' as plugin;
import 'package:over_react_analyzer_plugin/src/diagnostic_contributor.dart';

// TODO can you only create outlines for existing elements??
class ReactElementOutlineContributor implements OutlineContributor {
  final AnalyzerConverter converter = AnalyzerConverter();

  @override
  void computeOutline(OutlineRequest request, OutlineCollector collector) {
    if (request is! DartOutlineRequest) return;
    if (!request.path.contains('example')) return;

    final result = (request as DartOutlineRequest).result;
//
//    // need at least one outline for some reason
//    final element = new plugin.Element(plugin.ElementKind.UNKNOWN, '', 0);
//    // Outlines work intermittently, less so when this element does not constitute the whole file
//    collector.startElement(element, result.unit.offset, result.unit.length);
//    collector.endElement();

    result.unit.declaredElement.lineInfo;

    final elementEndStack = <int>[];

    final visitor = ComponentUsageVisitor((usage) {
      {
        final start = usage.node.argumentList.offset;
        final end = usage.node.argumentList.end;

        while (elementEndStack.isNotEmpty && start >= elementEndStack.last) {
          collector.endElement();
          elementEndStack.removeLast();
        }

        // USE FUNCTION_INVOCATION instead?
        final usageElement = plugin.Element(plugin.ElementKind.FUNCTION_INVOCATION, usage.builder.toSource(), 0);

        collector.startElement(usageElement, start, end - start);
        elementEndStack.add(end);
      }
    });
    result.unit.accept(visitor);

    if (elementEndStack.isEmpty) {
      // need at least one outline for some reason
      final element = plugin.Element(plugin.ElementKind.UNKNOWN, '', 0);
      // Outlines work intermittently, less so when this element does not constitute the whole file
      collector.startElement(element, result.unit.offset, result.unit.length);
      collector.endElement();
    } else {
      while (elementEndStack.isNotEmpty) {
        elementEndStack.removeLast();
        collector.endElement();
      }
    }
  }
}

==> over_react-4.8.0.tar.gz: tools/analyzer_plugin/lib/src/outline/element_outline.dart <==
import 'package:analyzer_plugin/utilities/analyzer_converter.dart';
import 'package:analyzer_plugin/utilities/outline/outline.dart';
import 'package:analyzer_plugin/protocol/protocol_common.dart' as plugin;
import 'package:over_react_analyzer_plugin/src/diagnostic_contributor.dart';

// TODO can you only create outlines for existing elements??
class ReactElementOutlineContributor implements OutlineContributor {
  final AnalyzerConverter converter = AnalyzerConverter();

  @override
  void computeOutline(OutlineRequest request, OutlineCollector collector) {
    if (request is! DartOutlineRequest) return;
    if (!request.path.contains('example')) return;

    final result = request.result;
//
//    // need at least one outline for some reason
//    final element = new plugin.Element(plugin.ElementKind.UNKNOWN, '', 0);
//    // Outlines work intermittently, less so when this element does not constitute the whole file
//    collector.startElement(element, result.unit.offset, result.unit.length);
//    collector.endElement();

    final elementEndStack = <int>[];

    final visitor = ComponentUsageVisitor((usage) {
      {
        final start = usage.node.argumentList.offset;
        final end = usage.node.argumentList.end;

        while (elementEndStack.isNotEmpty && start >= elementEndStack.last) {
          collector.endElement();
          elementEndStack.removeLast();
        }

        // USE FUNCTION_INVOCATION instead?
        final usageElement = plugin.Element(plugin.ElementKind.FUNCTION_INVOCATION, usage.builder.toSource(), 0);

        collector.startElement(usageElement, start, end - start);
        elementEndStack.add(end);
      }
    });
    result.unit!.accept(visitor);

    if (elementEndStack.isEmpty) {
      // need at least one outline for some reason
      final element = plugin.Element(plugin.ElementKind.UNKNOWN, '', 0);
      // Outlines work intermittently, less so when this element does not constitute the whole file
      collector.startElement(element, result.unit!.offset, result.unit!.length);
      collector.endElement();
    } else {
      while (elementEndStack.isNotEmpty) {
        elementEndStack.removeLast();
        collector.endElement();
      }
    }
  }
}
